"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _actions = require("./actions");

/**
 * Catch emit action for socket and prevent it from reaching reducer or latest middleware
 * @param {Object} socket socket connected with store
 * @returns {Function} which handle next middleware
 */
var socketMiddleware = function socketMiddleware(socket) {
  return function () {
    return function (next) {
      return function (action) {
        if (action.type !== _actions.EMIT_MESSAGE) {
          return next(action);
        }
        var _action$payload = action.payload,
            eventType = _action$payload.eventType,
            message = _action$payload.message;

        socket.emit(eventType, message);
      };
    };
  };
};

exports.default = socketMiddleware;